"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts_packet_1 = require("./ts-packet");
var ts_pes_1 = require("./ts-pes");
var PESPacketizer = /** @class */ (function () {
    function PESPacketizer() {
    }
    PESPacketizer.packetize = function (pes, transport_error_indicator, transport_priority, pid, transport_scrambling_control, continuity_counter, pts, dts) {
        var result = [];
        var id = ts_pes_1.stream_id(pes);
        var has_flag = (id !== 0xBC) && (id !== 0xBE) && (id !== 0xBF) && (id !== 0xF0) && (id !== 0xF1) && (id !== 0xFF) && (id !== 0xF2) && (id !== 0xF8);
        var PES_header_data_length = has_flag ? pes[ts_pes_1.PES_HEADER_SIZE + 2] : 0;
        var begin = has_flag ? (ts_pes_1.PES_HEADER_SIZE + 3 + PES_header_data_length) : ts_pes_1.PES_HEADER_SIZE;
        if (pts == null && dts != null) {
            pts = dts;
        }
        if (pts == null) {
            pts = ts_pes_1.PTS(pes);
        }
        if (dts == null) {
            dts = ts_pes_1.DTS(pes);
        }
        if (!has_flag) {
            pts = undefined;
            dts = undefined;
        }
        var new_PTS = Buffer.from(pts != null ? [
            ((pts != null ? 32 : 0) | (dts != null ? 16 : 0) | (((pts / (1 << 30)) & 0x7) << 1) | 1),
            ((((pts >>> 0) & 0x3FC00000) >> 22)),
            ((((pts >>> 0) & 0x003F8000) >> 15) << 1) | 1,
            ((((pts >>> 0) & 0x00007F80) >> 7)),
            ((((pts >>> 0) & 0x0000007F) >> 0) << 1) | 1,
        ] : []);
        var new_DTS = Buffer.from(dts != null ? [
            ((dts != null ? 16 : 0) | (((dts / (1 << 30)) & 0x7) << 1) | 1),
            ((((dts >>> 0) & 0x3FC00000) >> 22)),
            ((((dts >>> 0) & 0x003F8000) >> 15) << 1) | 1,
            ((((dts >>> 0) & 0x00007F80) >> 7)),
            ((((dts >>> 0) & 0x0000007F) >> 0) << 1) | 1,
        ] : []);
        var new_PES_header_data_length = new_PTS.length + new_DTS.length;
        pes = Buffer.concat([
            pes.slice(0, ts_pes_1.PES_HEADER_SIZE),
            Buffer.concat(has_flag ? [
                Buffer.from([(pts != null ? 128 : 0) | (dts != null ? 128 : 0), 0]),
                Buffer.from([new_PES_header_data_length]),
                new_PTS,
                new_DTS,
            ] : []),
            pes.slice(begin)
        ]);
        pes[4] = ((pes.length - ts_pes_1.PES_HEADER_SIZE) & 0xFF00) >> 8;
        pes[5] = ((pes.length - ts_pes_1.PES_HEADER_SIZE) & 0x00FF) >> 0;
        for (var i = 0; i < pes.length; i += ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE) {
            var payload = pes.slice(i, Math.min(pes.length, i + 184));
            var header = Buffer.from([
                ts_packet_1.SYNC_BYTE,
                ((transport_error_indicator ? 1 : 0) << 7) | ((begin === 0 ? 1 : 0) << 6) | ((transport_priority ? 1 : 0) << 5) | ((pid & 0x1F00) >> 8),
                (pid & 0x00FF),
                (transport_scrambling_control << 6) | ((ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE) > payload.length ? 0x30 : 0x10) | (continuity_counter & 0x0F),
            ]);
            continuity_counter = (continuity_counter + 1) & 0x0F;
            var packet = Buffer.concat([
                header,
                Buffer.from((payload.length < (ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE))
                    ? [(ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE - 1) - payload.length]
                    : []),
                Buffer.from((payload.length < (ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE - 1))
                    ? [0x00]
                    : []),
                ((payload.length < (ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE - 2))
                    ? Buffer.alloc((ts_packet_1.PACKET_SIZE - ts_packet_1.HEADER_SIZE - 2) - payload.length, 0xFF)
                    : Buffer.from([])),
                payload,
            ]);
            result.push(packet);
        }
        return result;
    };
    return PESPacketizer;
}());
exports.default = PESPacketizer;
