"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts_packet_1 = require("./ts-packet");
var ts_pes_1 = require("./ts-pes");
var is_complete_pes = function (pes) {
    if (ts_pes_1.PES_packet_length(pes) === 0) {
        return false;
    }
    return (ts_pes_1.PES_HEADER_SIZE + ts_pes_1.PES_packet_length(pes) - pes.length) <= 0;
};
var PESQueue = /** @class */ (function () {
    function PESQueue() {
        this.queue = [];
        this.partialPES = Buffer.from([]);
    }
    PESQueue.prototype.push = function (packet) {
        var begin = ts_packet_1.HEADER_SIZE + (ts_packet_1.has_adaptation_field(packet) ? 1 + ts_packet_1.adaptation_field_length(packet) : 0);
        if (this.partialPES.length === 0 && !ts_packet_1.payload_unit_start_indicator(packet)) {
            return;
        }
        if (ts_packet_1.payload_unit_start_indicator(packet)) {
            if (this.partialPES.length !== 0 && ts_pes_1.PES_packet_length(this.partialPES) === 0) {
                this.queue.push(this.partialPES);
            }
            var partial = packet.slice(begin);
            var packet_length = ts_pes_1.PES_packet_length(partial) === 0 ? Number.POSITIVE_INFINITY : ts_pes_1.PES_packet_length(partial);
            var next = begin + Math.max(0, ts_pes_1.PES_HEADER_SIZE + packet_length);
            this.partialPES = packet.slice(begin, next);
        }
        else {
            var packet_length = ts_pes_1.PES_packet_length(this.partialPES) === 0 ? Number.POSITIVE_INFINITY : ts_pes_1.PES_packet_length(this.partialPES);
            var next = begin + Math.max(0, ts_pes_1.PES_HEADER_SIZE + packet_length - this.partialPES.length);
            this.partialPES = Buffer.concat([this.partialPES, packet.slice(begin, next)]);
        }
        if (is_complete_pes(this.partialPES)) {
            this.queue.push(this.partialPES);
            this.partialPES = Buffer.from([]);
        }
    };
    PESQueue.prototype.pop = function () {
        return this.queue.shift();
    };
    PESQueue.prototype.isEmpty = function () {
        return this.queue.length === 0;
    };
    PESQueue.prototype.clear = function () {
        this.partialPES = Buffer.from([]);
        this.queue.length = 0;
    };
    return PESQueue;
}());
exports.default = PESQueue;
