"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DTS = exports.PTS = exports.has_DTS = exports.has_PTS = exports.has_flags = exports.PES_packet_length = exports.stream_id = exports.packet_start_code_prefix = exports.PES_HEADER_SIZE = void 0;
exports.PES_HEADER_SIZE = 6;
var packet_start_code_prefix = function (pes) {
    return (pes[0] << 16) | (pes[1] << 8) | pes[2];
};
exports.packet_start_code_prefix = packet_start_code_prefix;
var stream_id = function (pes) {
    return pes[3];
};
exports.stream_id = stream_id;
var PES_packet_length = function (pes) {
    return (pes[4] << 8) | pes[5];
};
exports.PES_packet_length = PES_packet_length;
var has_flags = function (pes) {
    var id = exports.stream_id(pes);
    return (id !== 0xBC) && (id !== 0xBE) && (id !== 0xBF) && (id !== 0xF0) && (id !== 0xF1) && (id !== 0xFF) && (id !== 0xF2) && (id !== 0xF8);
};
exports.has_flags = has_flags;
var has_PTS = function (pes) {
    if (!exports.has_flags(pes)) {
        return false;
    }
    return (pes[exports.PES_HEADER_SIZE + 1] & 0x80) !== 0;
};
exports.has_PTS = has_PTS;
var has_DTS = function (pes) {
    if (!exports.has_flags(pes)) {
        return false;
    }
    return (pes[exports.PES_HEADER_SIZE + 1] & 0x40) !== 0;
};
exports.has_DTS = has_DTS;
var PTS = function (pes) {
    if (!exports.has_PTS(pes)) {
        return undefined;
    }
    var value = 0;
    value *= (1 << 3);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + 0] & 0x0E) >> 1);
    value *= (1 << 8);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + 1] & 0xFF) >> 0);
    value *= (1 << 7);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + 2] & 0xFE) >> 1);
    value *= (1 << 8);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + 3] & 0xFF) >> 0);
    value *= (1 << 7);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + 4] & 0xFE) >> 1);
    return value;
};
exports.PTS = PTS;
var DTS = function (pes) {
    if (!exports.has_DTS(pes)) {
        return undefined;
    }
    var offset = exports.has_PTS(pes) ? 5 : 0;
    var value = 0;
    value *= (1 << 3);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + offset + 0] & 0x0E) >> 1);
    value *= (1 << 8);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + offset + 1] & 0xFF) >> 0);
    value *= (1 << 7);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + offset + 2] & 0xFE) >> 1);
    value *= (1 << 8);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + offset + 3] & 0xFF) >> 0);
    value *= (1 << 7);
    value += ((pes[exports.PES_HEADER_SIZE + 3 + offset + 4] & 0xFE) >> 1);
    return value;
};
exports.DTS = DTS;
